
/*
 * Copyright, 2006-2007 Rick Gorton	All Rights Reserved
 *
 * wslc.h - declarations for Web Site Consistency Checker (WSLC)
 *
 * Contents:
 *      constants, enums, structs
 *      common macros
 *      function prototypes
 */

enum FLAGS_VALS {
	FLAGS_UNSEEN		= 0x0000,
	FLAGS_FILE_PROCESSED	= 0x0001,
	FLAGS_BEING_ANALYZED	= 0x0002,
	FLAGS_FILE_REFERENCED	= 0x0004,
	FLAGS_LAST
	};

enum WSLC_TYPE {
	WSLC_UNKNOWN = 1,
	WSLC_ERROR,	// problem accessing the file
	WSLC_SDP,	// sockets, devices, pipes
	WSLC_SETX,	// setuid, setgid
	WSLC_LINK,	// symbolic link
	WSLC_DIR,	// directory
	WSLC_SCRIPT,	// shell script "#!"
	WSLC_ELF,	// ELF executable
	WSLC_GRAPHICS,	// JPEG, JPG, 
	WSLC_HTML,	// .htm, .html files
	WSLC_DOCUMENT,	// ms office, pdf
	WSLC_GZ_COMP,	// gzipped or compressed
	WSLC_LAST
	};

enum IERR_CODES {
        IERR_MALLOC = 1,// probably ENOMEM
        IERR_REALPATH,  // realpath() passed junk
        IERR_HASHTAB,   // corrupted hash table
        IERR_LAST
        };

typedef struct IERR_STRING {
        enum IERR_CODES code;
        const char      *string;
        } IERR_STRING;

static IERR_STRING IntErrString[IERR_LAST] = {
        { IERR_MALLOC,  "malloc error (probably ENOMEM)" },
        { IERR_REALPATH,"realpath() problem" },
        { IERR_HASHTAB, "File not in internal tables" },
        { IERR_LAST,    "Unknown error - missing a description for this error" }
        };

#define ANCHOR_DEF	0x1
#define	ANCHOR_REF	0x2
typedef struct	ANL {
    void	*next;
    long	defref;
    char	aname[PATH_MAX];
    } ANL;

typedef struct FILREC {
    void		*next;
    ANL			*AncList;
    long		hash;
    enum WSLC_TYPE	type;
    enum FLAGS_VALS	flags;
    char		filepath [PATH_MAX];
    } FILREC;
// NCHAINS should be a power of two
#define NCHAINS	256

typedef struct HFLIST {
    void	*next;
    FILREC	*FRec;
    } HFLIST;

extern HFLIST *HF_l;
extern FILREC *HTable[NCHAINS];

typedef struct FILEDATA {
    void        *next;
    char        ht_file[PATH_MAX];
    int         ht_lineno;
    char        ref_file[PATH_MAX];
    } FILEDATA;

extern FILEDATA *ExRefs;

// Function prototypes
void Error (char *Emsg);
void ErrorD (char *fmts, int var);
void ErrorS (char *fmts, char *var);
long sdbm_hash (char *str, int len);
void AddFileToReferencedList (char *ref_htfile, int *lineno, char *the_f);
char *GetNextScanPoint (char *bufstart, int *lineno);
FILREC *FindRecByName (char *fname);
enum WSLC_TYPE GetFileType (FILREC *frec);
enum WSLC_TYPE GuessFileType (char *Fname, int FnameLen);
void ReportTypeStats();
int AddFileToFullFileList (char *file);
void RecurseWalkDir (const char *name);
void ReportSiteError (char *msg);
void InternalError (const char *func, int eoff, enum IERR_CODES code);
void AddFileDatum (FILEDATA **ListName, char *hfile, int *lineno, char *HRefPath);
